@@ -87,6 +87,7 @@ typedef struct wavstuff {
     unsigned short samplesPerBlock;
     unsigned short blockAlign;
     st_size_t dataStart;  /* need to for seeking */
+    int found_cooledit;
     
     /* following used by *ADPCM wav files */
     unsigned short nCoefs;          /* ADPCM: number of coef sets */
@@ -402,17 +403,23 @@ static st_ssize_t findChunk(ft_t ft, const char *Label)
     {
         if (st_reads(ft, magic, 4) == ST_EOF)
         {
-            st_fail_errno(ft,ST_EHDR,"WAVE file has missing %s chunk", Label);
+            st_fail_errno(ft, ST_EHDR, "WAVE file has missing %s chunk", 
+                          Label);
             return ST_EOF;
         }
         st_readdw(ft, &tmp_len);
         len = tmp_len;
-        st_report("Chunk %s",magic);
-        if (strncmp(Label, magic, 4) == 0)
-            break;              /* Found the data chunk */
+        st_report("WAV Chunk %s", magic);
 
-        
-        st_seek(ft, len, SEEK_CUR);     /* skip to next chunk */
+        if (strncmp(Label, magic, 4) == 0)
+            break; /* Found the data chunk */
+
+        /* skip to next chunk */
+        if (st_seek(ft, len, SEEK_CUR) != ST_SUCCESS)
+        {
+            st_fail_errno(ft,ST_EHDR, "WAV chunk appears to have invalid size %d.", len);
+            return ST_EOF;
+        }
     }
     return len;
 }
@@ -834,7 +841,11 @@ int st_wavstartread(ft_t ft)
 
     /* Now look for the wave data chunk */
     dwDataLength = len = findChunk(ft, "data");
-    /* findChunk() only returns if chunk was found */
+    if (len == ST_EOF)
+    {
+        st_fail_errno(ft, ST_EOF, "Could not find data chunk.");
+        return ST_EOF;
+    }
 
     /* Data starts here */
     wav->dataStart = st_tell(ft);
@@ -909,13 +920,16 @@ int st_wavstartread(ft_t ft)
 
     /* Horrible way to find Cool Edit marker points. Taken from Quake source*/
     ft->loops[0].start = -1;
+    wav->found_cooledit = 0;
     if(ft->seekable){
         /*Got this from the quake source.  I think it 32bit aligns the chunks 
          * doubt any machine writing Cool Edit Chunks writes them at an odd 
          * offset */
         len = (len + 1) & ~1;
-        st_seek(ft, len, SEEK_CUR);
-        if( findChunk(ft, "LIST") != ST_EOF){
+        if (st_seek(ft, len, SEEK_CUR) == ST_SUCCESS &&
+            findChunk(ft, "LIST") != ST_EOF)
+        {
+            wav->found_cooledit = 1;
             ft->comment = (char*)malloc(256);
             /* Initialize comment to a NULL string */
             ft->comment[0] = 0;
@@ -924,10 +938,20 @@ int st_wavstartread(ft_t ft)
                 if (st_reads(ft,magic,4) == ST_EOF)
                     break;
 
-                if (strncmp(magic,"INFO",4) == 0)
+                /* First look for type fields for LIST Chunk and
+                 * skip those if found.  Since a LIST is a list
+                 * of Chunks, treat the remaining data as Chunks
+                 * again.
+                 */
+                if (strncmp(magic, "INFO", 4) == 0)
                 {
                     /*Skip*/
-                    st_report("Chunk INFO");
+                    st_report("Type INFO");
+                }
+                else if (strncmp(magic, "adtl", 4) == 0)
+                {
+                    /* Skip */
+                    st_report("Type adtl");
                 }
                 else
                 {
@@ -983,6 +1007,8 @@ int st_wavstartread(ft_t ft)
                         st_report("Chunk ltxt");
                         st_readdw(ft,&dwLoopPos);
                         ft->loops[0].length = dwLoopPos - ft->loops[0].start;
+                        if (len > 4)
+                           st_seek(ft, len - 4, SEEK_CUR); 
                     } 
                     else 
                     {
@@ -1024,7 +1050,8 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
              * wav handler.  Sometimes it accounts for stereo,
              * sometimes it does not.
              */
-            if (len > (wav->numSamples*ft->info.channels)) 
+            /* See reason for cooledit check in comments below */
+            if (wav->found_cooledit && len > (wav->numSamples*ft->info.channels)) 
                 len = (wav->numSamples*ft->info.channels);
 
             done = 0;
@@ -1073,7 +1100,8 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
 
 #ifdef ENABLE_GSM
         case ST_ENCODING_GSM:
-            if (len > wav->numSamples) 
+            /* See reason for cooledit check in comments below */
+            if (wav->found_cooledit && len > wav->numSamples) 
                 len = wav->numSamples;
 
             done = wavgsmread(ft, buf, len);
@@ -1082,10 +1110,18 @@ st_ssize_t st_wavread(ft_t ft, st_sample_t *buf, st_ssize_t len)
         break;
 #endif
         default: /* assume PCM or float encoding */
-#if 0
-            if (len > wav->numSamples) 
+            /* Cooledit seems to put a non-standard IFF LIST at
+             * the end of the file.  When this is detected,
+             * go ahead and only read in the reported size
+             * of data chunk so the LIST data is not treated
+             * as audio.  
+             * In other cases, go ahead and read unit EOF
+             * This allows us to process WAV files that are
+             * greater then 2Gig and can't be represented
+             * by the 32-bit size field.
+             */
+            if (wav->found_cooledit && len > wav->numSamples) 
                 len = wav->numSamples;
-#endif
 
             done = st_rawread(ft, buf, len);
             /* If software thinks there are more samples but I/O */
